//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR RESEARCH, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// Dictionary.cc	Holds a set of words in a dictionary structure.
//
// Author:  Ken Ronny Schouten

#include <string.h>
#include <stdlib.h>
#include "Dictionary.h"
#include "lang.h"
#include "Index.h"

extern int memb;

int AllocT[] = { 20, 8, 4, 3, 2 };
int ReallocT[] = { 10, 10, 10, 5, 5, 5, 5, 5, 5, 5, 4, 4, 2 };

Dictionary::Dictionary()
{
    for (int i = 0 ; i < 256; i++) {
	_elements[i].letter = i;
	_elements[i].nextletter = NULL;
	_elements[i]._url = new URLPTRSplaySet;
	if (_elements[i]._url == NULL) {
	    cout.form(AllocMemory);
	    exit(1);
	}
    }
}

int Dictionary::add(unsigned char *word, Url *url)
{
    int    len  = 1;
    struct _array   *array;
    struct _element *element;
    int left, right, ind, loop, num;
    int length = strlen((char *) word);
    int allocl = sizeof(AllocT)/sizeof(int);
    int reallocl = sizeof(ReallocT)/sizeof(int);

    element = &(_elements[word[0]]);

    while (len < length) {
	if (element->nextletter == NULL) {

	    // No array allocated, allocate it
	    element->nextletter = (struct _array *)
		malloc(sizeof(struct _array));
	    
	    if (element->nextletter == NULL) {
		cout.form(AllocMemory);		
		exit(1);
	    }
	    
	    array = element->nextletter;

	    if (len <= allocl)
		num = AllocT[len-1];
	    else num = 1;

	    unsigned size = sizeof(struct _element) * num;
	    array->cells = (struct _element *) malloc(size);
	    
	    if (array->cells == NULL) {
		cout.form(AllocMemory);				
		exit(1);
	    }
	    
	    array->cells->letter = word[len];
	    array->cells->nextletter = NULL;
	    array->cells->_url = new URLPTRSplaySet;

	    if (array->cells->_url == NULL) {
		cout.form(AllocMemory);				
		exit(1);
	    }
	    
	    array->length = num;
	    array->num = 1;
	    element = array->cells;
	    
	} else {
	    // Array present, binary search
	    array = element->nextletter;

	    loop = 1;
	    left = 0;
	    right = array->num - 1;
	    
	    while (loop) {

		ind = (int) (left + right)/2;
	    
		if (left >= right)
		    loop = 0;

		element = array->cells+ind;
		if (element->letter == word[len]) {
		    loop = 0;
		} else if (element->letter > word[len])
		    right = ind - 1;
		else
		    left = ind + 1;
	    }

	    // If the letter is not found in the array, we need to add it
	    // and that may also mean that we need to realloc the structure.
	    if (element->letter != word[len]) {
		// Do we need to realloc ?
		if (array->length - array->num < 1) {
		    if (len <= reallocl)
			num = array->length + ReallocT[len-1];
		    else num = array->length + 1;                   

		    array->cells = (struct _element *)
				realloc((struct _element *) array->cells,
					(int) sizeof(struct _element)*num);

		    if (array->cells == NULL) {
			cout.form(AllocMemory);
			exit(1);
		    }		    

		    array->length = num;
		    element = array->cells+ind;
		}
		
		if (element->letter < word[len]) {
		    ind++;
		    element++;
		}

		if (ind < array->num) {
		    memmove((struct _element *) element+1,
			    (struct _element *) element,
			    sizeof(struct _element)*(array->num-ind));
		}
		
		element->letter = word[len];
		element->nextletter = NULL;
		element->_url = new URLPTRSplaySet;

		if (element->_url == NULL) {
		    cout.form(AllocMemory);		    
		    exit(1);
		}		    		

		array->num++;
	    }
	}
	len++;
    }

    Pix t = element->_url->add(url);

    return 0;
}

Dictionary::travers(Disk* disk)
{
    struct _array   *array;
    unsigned char   word[256];

    for (int i = 0; i < 256; i++) {

	array = _elements[i].nextletter;
	word[0] = i;
	word[1] = '\0';

	if ((_elements[i]._url->length()) != 0) {
	    Entry entry(word);
	    entry._urllist = *(_elements[i]._url);
	    disk->StoreEntry(entry);
	}
	
	if (array != NULL)
	    travers(disk, array, word, 1);
    }
	
    return 0;
}

Dictionary::travers(Disk* disk, struct _array *array,
		    unsigned char *word, int num)
{
    struct _element *element;
    int len = 0;

    element = array->cells;

    while (len < array->num) {

	word[num] = element->letter;
	word[num+1] = '\0';

	if ((element->_url->length()) != 0) {
	    Entry entry(word);
	    entry._urllist = *(element->_url);
	    disk->StoreEntry(entry);
	}

	if (element->nextletter != NULL) {
	    travers(disk, element->nextletter, word, num+1);
	}

	element++;
	len++;
    }

    return 0;
}
