//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR RESEARCH, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// Wordfilter.cc	Wordfilter source. Builds a set of words that
//			should not be indexed (the, a, and, or...).
//
// Author:  Ken Ronny Schouten

#include <std.h>
#include <ctype.h>
#include "Wordfilter.h"
#include "lang.h"

#define True (1)
#define False (0)

WordFilterParams default_params = {
    0,		// minimum permissible word length
    32767,	// maximum permissible word length
    1,		// allow numbers
    1,		// allow strings with all identical chars
};

Wordfilter::Wordfilter()
{
    params = default_params;
}

Wordfilter::Wordfilter(WordFilterParams &wfp)
{
    params = wfp;
}

Wordfilter::Wordfilter(String& filename, WordFilterParams &wfp)
{
    filebuf fin;
    
    if ((fin.open(filename, _IOS_INPUT)) == 0) {
	cerr.form(OpenFile, (char *) filename);
    }	
    istream file(&fin);

    String string;
    
    file >> string;
    while(file) {
	_words.add(string);
	file >> string;
    }

    params = wfp;
}

int Wordfilter::add(String& filename)
{
    filebuf fin;
    
    if ((fin.open(filename, _IOS_INPUT)) == 0) {
	cerr.form(OpenFile, (char *) filename);
	exit(1);
    }	
    istream file(&fin);

    String string;
    
    file >> string;
    while(file) {
	_words.add(string);
	file >> string;
    }

    return 0;
}

// This procedure returns true if the word is in the wordfilter, thus
// it is NOT allowed. On any other situations it will return false
int Wordfilter::IsNotAllowed(String &string)
{
    int slen = string.length();

    // Check string lenghts
    if ( slen < params.min_word_length ) return True;
    if ( slen > params.max_word_length ) return True;

    // Check for numbers
    if ( !params.allow_numbers ) {
	int i = 0;
	if ( string[i] == '-' || string[i] == '+' ) i++;
	for ( ; i<slen && isdigit(string[i]); i++ ) ;
	if ( i == slen ) return True;
    }

    // Check for strings with all same char
    if ( !params.allow_identicals ) {
	int i;
	int ch = string[0];
	for ( i=1; i<slen && string[i]==ch; i++ ) ;
	if ( i == slen ) return True;
    }
	
    return _words.contains(string);
}

