//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR RESEARCH, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// TermNode.h	TermNode class for the syntax tree
//
// Author: Haiyan Yang

#ifndef _TERMNODE_H
#define _TERMNODE_H

#include <iostream.h>
#include <String.h>
#include <UP.SplaySet.h>
#include "SetNode.h"



class TermNode
{
public:
    friend ostream& operator<<(ostream& os, TermNode& n);
    virtual ostream& print(ostream&) = 0;
    virtual SetNode* build() = 0;
    virtual int isNot() = 0;
};


class OrTermNode : public TermNode
{
private:
    TermNode*	_left;
    TermNode*	_right;

public:
    OrTermNode(TermNode* left, TermNode* right);
    SetNode* build();
    int isNot();
    ostream& print(ostream& os);
};

class AndTermNode : public TermNode
{
private:
    TermNode*	_left;
    TermNode*	_right;

public:
    AndTermNode(TermNode* left, TermNode* right);
    SetNode* build();
    int isNot();
    ostream& print(ostream& os);
};

class NotTermNode : public TermNode
{
private:
    TermNode*	_child;

public:
    NotTermNode(TermNode* child);
    SetNode* build();
    int isNot();
    ostream& print(ostream& os);
};

class TextTermNode : public TermNode
{
private:	
    unsigned char * _str;

public:
    TextTermNode(unsigned char *str);
    SetNode* build();
    int isNot();
    ostream& print(ostream& os);
};

inline OrTermNode::OrTermNode(TermNode* left, TermNode* right)
{
    _left = left; 
    _right = right;
}

inline int OrTermNode::isNot() 
{ 
    return 0; 
}

inline ostream& OrTermNode::print(ostream& os)
{ 
    return os << "Or(" << *_left << ", " << *_right << ")"; 
}


inline AndTermNode::AndTermNode(TermNode* left, TermNode* right)
{
    _left = left; 
    _right = right;
}

inline int AndTermNode::isNot() 
{ 
    return 0; 
}

inline ostream& AndTermNode::print(ostream& os)
{ 
    return os << "And(" << *_left << ", " << *_right << ")"; 
}


inline NotTermNode::NotTermNode(TermNode* child)
{
    _child = child;
}

inline int NotTermNode::isNot() 
{ 
    return 1; 
}

inline ostream& NotTermNode::print(ostream& os)
{ 
    return os << "Not(" << *_child << ")"; 
}


inline TextTermNode::TextTermNode(unsigned char *str)
{
    _str = str;
}

inline int TextTermNode::isNot() 
{ 
    return 0; 
}

inline ostream& TextTermNode::print(ostream& os)
{ 
    return os << '"' << _str << '"'; 
}


#endif /* TERMNODE_H */
