//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR RESEARCH, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// main.cc	The main routine for the FFW search engine.
//
// Author: Haiyan Yang
//
// Changes: January 1995 Brd Hfjeld
//				Added support for date searches
//				Added search hit limits
//	    May 1995 Brd Hfjeld
//				Fixed option parsing to allow
//				searching for words starting with
//				hyphen.
//
#include <iostream.h>
#include <strstream.h>
#include <String.h>
#include <S.SplaySet.h>
#include <Disk.h>
#include <lang.h>
#include "Parser.h"
#include <time.h>

// Forward declarations
int CheckDate(struct tm *tm);

// Stuff..
#define True (1)
#define False (0)

int main(int argc, char *argv[])
{
    StringSplaySet	indexFiles;
    char *fname;
    struct tm tm;
    int debug = False;

    // Default values for search parameters
    struct search_params sp = {
	-2147483647, 		// after -2**31+1
	2147483647, 		// before 2**31-1
	2147483647,		// Max internal
	2147483647,		// Max output
    };
	
    // Absolutely no arguments specified -> print help for poor user
    if (argc <= 1) {
	cout.form(SearchHelp, argv[0]);
	exit(1);
    }

    // Set up time system to be GMT for mktime()
    putenv("TZ=GMT");
    tzset();

    // Parse commandline
    int argi;
    for ( argi=1; argi<argc; argi++ ) 
	if ( strcmp(argv[argi], "-i") == 0 ) {	// Index database
	    if ( argi+1 < argc ) indexFiles.add(String(argv[++argi]));
	} else if (strcmp(argv[argi], "-a") == 0) {
	    if ( argi+1 >= argc ) continue;
	    memset(&tm, 0, sizeof(struct tm));
	    sscanf(argv[++argi], "%d-%d-%d-%d-%d-%d",
		   &tm.tm_year, &tm.tm_mon, &tm.tm_mday,
		   &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
	    tm.tm_mon--;
	    tm.tm_year -= 1900;
	    if ( !CheckDate(&tm) ) {
		cout.form(BadAfterDate);
		exit(1);
	    }
	    sp.after = mktime(&tm);
	} else if (strcmp(argv[argi], "-b") == 0) {
	    if ( argi+1 >= argc ) continue;
	    memset(&tm, 0, sizeof(struct tm));
	    sscanf(argv[++argi], "%d-%d-%d-%d-%d-%d",
		   &tm.tm_year, &tm.tm_mon, &tm.tm_mday,
		   &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
	    tm.tm_mon--;
	    tm.tm_year -= 1900;
	    if ( !CheckDate(&tm) ) {
		cout.form(BadBeforeDate);
		exit(1);
	    }
	    sp.before = mktime(&tm);
	} else if (strcmp(argv[argi], "-w") == 0) {
	    if ( argi+1 < argc ) sp.max_internal = atoi(argv[++argi]);
	} else if (strcmp(argv[argi], "-o") == 0) {
	    if ( argi+1 < argc ) sp.max_output = atoi(argv[++argi]);
	} else if (strcmp(argv[argi], "-d") == 0) {
	    debug = True;
	} else {
	    // Start of expression
	    break;
	}

    // No search term specified?
    if ( argi >= argc ) {
	cout.form(NoSearchTerm);
	exit(1);
    }

    // No index files specified?
    if (indexFiles.length() < 1) {
	cout.form(NoIndexFiles);
	exit(1);
    }


    // First create a string from the what's left to parse of the
    // commandline (which happens to be the search term).
    String searchTerm;
    for (; argi < argc; argi++) {
	searchTerm += argv[argi];
	if (argi < argc-1)
	    searchTerm += " ";
    }

    QueryParser qp;
    SetNode *tree = qp.parse(searchTerm);

    if (debug) {
	// Dump the set expression to stdout
    	cout << "Parsed output is:" << endl;
    	tree->print(cout);
    	cout << endl;
    	cout.flush();
	exit(0);
    }


    // Evaluate the result and put it in result
    URLSet result;
    for (Pix fileIter = indexFiles.first();
	 fileIter != 0;
	 indexFiles.next(fileIter)) {
	Disk d((unsigned char *)((char *)indexFiles(fileIter)), DREAD);
	switch ((d.error())) {
	  case DOK:
	    break;
	  case DOPEN:
	    cout.form(OpenFile, (char *)indexFiles(fileIter));
	    exit(1);
    	  case DBAD:
	    cout.form(BadIndex, (char *)indexFiles(fileIter));
	    exit(1);
	  default:
	    cout.form(UnknownError, d.error(), "Disk::Disk()");
	    exit(1);	    
	}
	result |= tree->evaluate(d, &sp);
	if ( result.length() > sp.max_internal ) {
	    cout.form(InternalExceeded, sp.max_internal);
	    exit(1);
	}
    }

    // If we exceed output limit inform the poor user
    if ( result.length() > sp.max_output ) 
	cout.form(OutputExceeded, sp.max_output, result.length());
    
    // And finally, output the result with one entry on each line
    // Stop writing if output limit was exceeded
    int out_ctr;
    Pix i;
    for (out_ctr = 0, i = result.first();
	 i != 0 && out_ctr<sp.max_output;
	 result.next(i), out_ctr++ ) {
	result(i)->print(); cout << endl;
    }

    // We exit the fast way.. don't want to spend time cleaning up
    exit(0);
}


//
// Check for valid date values
// The size of the time_t (== long) restricts year span
//

#define MAX_YRS 67	// 2**31/60/60/24/365

int CheckDate(struct tm *tm)
{
    if ( tm->tm_year < (70-MAX_YRS) || tm->tm_year > (70+MAX_YRS) ||
	 tm->tm_mon < 0 || tm->tm_mon > 11 ||
	 tm->tm_mday < 1 || tm->tm_mday > 31 ||
	 tm->tm_hour < 0 || tm->tm_hour > 23 ||
	 tm->tm_min < 0 || tm->tm_min > 59 ||
	 tm->tm_sec < 0 || tm->tm_sec > 59 )
	return False;

    return True;
}
